import hooks._error_handlers
import hooks._settings
import hooks._logs
from log_trace.decorators import trace


@trace
def add_hooks(app):
    hooks._error_handlers.add_hooks(app)
    hooks._settings.add_hooks(app)
    hooks._logs.add_hooks(app)



import os
import logging.config
import platform
import socket
import yaml
from eve import __version__ as eve_version
from cerberus import __version__ as cerberus_version
from werkzeug.utils import secure_filename

VERSION = '0.1.0'

# TODO: organize into functions!


def set_optional_setting(var):
    if os.environ.get(var):
        SETTINGS[var] = os.environ.get(var)


# set environment variables from _env.conf (which is in .gitignore)
if os.path.exists('_env.conf'):
    with open('_env.conf') as setting:
        for line in setting:
            if not line.startswith('#'):
                line = line.rstrip()
                nvp = line.split('=')
                if len(nvp) == 2:
                    os.environ[nvp[0].strip()] = nvp[1].strip()


# TODO: sanitize smtp/email_recipients
SETTINGS = {
    'ES_API_NAME': '{$project_name}',

    'ES_MONGO_HOST': os.environ.get('ES_MONGO_HOST', 'localhost'),
    'ES_MONGO_PORT': os.environ.get('ES_MONGO_PORT', 27017),
    'ES_MONGO_DBNAME': os.environ.get('ES_MONGO_DBNAME', '{$project_name}'),
    'ES_API_PORT': os.environ.get('ES_API_PORT', 2112),
    'ES_CMS_ADAPTER': os.environ.get('ES_CMS_ADAPTER', 'ConsoleOnly'),
    'ES_INSTANCE_NAME': os.environ.get('ES_INSTANCE_NAME', socket.gethostname()),
    'ES_TRACE_LOGGING': os.environ.get('ES_TRACE_LOGGING', 'Enabled'),
    'ES_PAGINATION_LIMIT': os.environ.get('ES_PAGINATION_LIMIT', 1000),
    'ES_PAGINATION_DEFAULT': os.environ.get('ES_PAGINATION_DEFAULT', 500),
    'SMTP_HOST': os.environ.get('SMTP_HOST', 'internalmail.cri.com'),
    'SMTP_PORT': os.environ.get('SMTP_PORT', 25),
    'EMAIL_RECIPIENTS': os.environ.get('EMAIL_RECIPIENTS', 'errors@cri.com, michael.ottoson@cri.com')
}

# optional settings...
set_optional_setting('ES_MONGO_USERNAME')
set_optional_setting('ES_MONGO_PASSWORD')
set_optional_setting('ES_MONGO_AUTH_SOURCE')
set_optional_setting('ES_MEDIA_BASE_URL')
set_optional_setting('ES_PUBLIC_RESOURCES')

# cancellable settings...
# if SETTINGS.get('ES_CANCELLABLE') == '':
#     del SETTINGS['ES_CANCELLABLE']


# Set up logging
API_NAME = SETTINGS.get('ES_API_NAME')
LOG_FOLDER = f'/var/log/{secure_filename(API_NAME)}'

if not os.path.exists(LOG_FOLDER):
    os.makedirs(LOG_FOLDER)

with open('logging.yml', 'r') as f:
    logging_config = yaml.load(f, Loader=yaml.FullLoader)

logging_config['handlers']['smtp']['mailhost'] = [SETTINGS.get('SMTP_HOST'), SETTINGS.get('SMTP_PORT')]
logging_config['handlers']['smtp']['toaddrs'] = [e.strip() for e in SETTINGS.get('EMAIL_RECIPIENTS').split(',')]
logging_config['handlers']['smtp']['subject'] = f'Problem encountered with {API_NAME}'
logging_config['handlers']['all']['filename'] = os.path.join(LOG_FOLDER, 'all.log')
logging_config['handlers']['warn']['filename'] = os.path.join(LOG_FOLDER, 'warn.log')
logging_config['handlers']['error']['filename'] = os.path.join(LOG_FOLDER, 'error.log')

logging.config.dictConfig(logging_config)

werkzeug_log = logging.getLogger('werkzeug')
werkzeug_log.setLevel(logging.ERROR)

LOG = logging.getLogger('configuration')


LOG.info('%s version:  %s', API_NAME, VERSION)
LOG.info('Eve version:      %s', eve_version)
LOG.info('Cerberus version: %s', cerberus_version)
LOG.info('Python version:   %s', platform.sys.version)

EMAIL_FORMAT = '''%(levelname)s sent from {0} instance "{1}" (hostname: {2})

%(asctime)s - %(levelname)s - File: %(filename)s - %(funcName)s() - Line: %(lineno)d -  %(message)s
'''.format(API_NAME, SETTINGS.get('ES_INSTANCE_NAME'), socket.gethostname())

EMAIL_FORMAT += f'''
{API_NAME} version:  {VERSION}
Eve version:      {eve_version}
Cerberus version: {cerberus_version}
Python version:   {platform.sys.version}

'''

for setting in sorted(SETTINGS):
    key = setting.upper()
    if ('PASSWORD' not in key) and ('SECRET' not in key):
        LOG.info('%s: %s', setting, SETTINGS[setting])
        EMAIL_FORMAT += '{0}: {1}\n'.format(setting, SETTINGS[setting])
EMAIL_FORMAT += '\n\n'

LOGGER = logging.getLogger()
HANDLERS = LOGGER.handlers

SMTP_HANDLER = [x for x in HANDLERS if x.name == 'smtp'][0]
SMTP_HANDLER.setFormatter(logging.Formatter(EMAIL_FORMAT))



import logging
from flask import jsonify, make_response

LOG = logging.getLogger('utils')


def make_error_response(message, code, issues=[], **kwargs):
    if 'exception' in kwargs:
        ex = kwargs.get('exception')
        LOG.exception(message, ex)

        issues.append({
            'exception': {
                'name': type(ex).__name__,
                'type': ".".join([type(ex).__module__, type(ex).__name__]),
                'args': ex.args
            }
        })

    resp = {
        '_status': 'ERR',
        '_error': {
            'message': message,
            'code': code
        }
    }

    if issues:
        resp['_issues'] = issues

    return make_response(jsonify(resp), code)





"""
Function decorators used in this project
- @trace - log_trace entries and exits of decorated function
"""
import sys
import traceback
import logging
from functools import wraps

from io import StringIO
from six import reraise
from configuration import SETTINGS

LOG = logging.getLogger('trace')


def _trace(target):
    """
    This is a decorator that log_trace when the decorated function is entered and exited.
    Any unhandled exceptions raised by the decorated function are logged and re-raised.
    """

    @wraps(target)
    def _wrapper(*args, **kwargs):
        global return_value
        LOG.trace(
            'Entering function: [{0}].[{1}]'
                .format(
                target.__module__,
                target.__name__
            )
        )

        try:
            return_value = target(*args, **kwargs)
        except Exception as ex:  # pylint: disable=broad-except
            exc_type, exc_value, tr_back = sys.exc_info()
            LOG.trace(
                'Exiting function: [%s].[%s] by unhandled exception "%s"',
                target.__module__,
                target.__name__,
                ex
            )
            sbuf = StringIO()
            traceback.print_stack(file=sbuf)
            sbuf.seek(0)
            LOG.trace(sbuf.read())
            reraise(exc_type, exc_value, tr_back)

        LOG.trace(
            'Exiting function: [{0}].[{1}] by return'
                .format(
                target.__module__,
                target.__name__
            )
        )

        return return_value

    return _wrapper


def _trace_disabled():
    pass


trace = _trace_disabled
if SETTINGS.get('ES_TRACE_LOGGING') == 'Enabled':
    trace = _trace



from glob import glob
from setuptools import setup

# TODO: pull PyInstaller out of packages, make it a command (mkwin32?)
# TODO: remove python 2.7 compatibilities (six, .format(), etc.)
# TODO: doco CORS

bin = glob('bin/*')

setup(
    name='eve-utils',
    version='0.6.4',
    description='Templates and scripts to rapidly spin up an Eve-based API.',
    long_description=open('README.rst').read(),
    license='MIT',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
        'Topic :: Software Development :: Code Generators',
        'Topic :: Utilities'
    ],
    url='http://www.pointw.com',
    author='Michael Ottoson',
    author_email='michael@pointw.com',
    packages=['eve_utils'],
    include_package_data=True,
    install_requires=[
        'libcst',
        'inflect'
    ],
    scripts=bin,
    zip_safe=False
)

